﻿using C1.WPF.FlexGrid;
using Microsoft.Ink;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Markup;
using System.Drawing;
using System.Windows.Media;
using System.Windows.Ink;

namespace FlexGrid.Ink.Input
{
    public class InputManager : DependencyObject, INotifyPropertyChanged
    {
        private CellRange editingCell;
        private ColoringCellFactory cellFactory;        
        public C1FlexGrid FlexGrid
        {
            get
            {
                return (C1FlexGrid)this.GetValue(FlexGridProperty);
            }
            set
            {
                this.SetValue(FlexGridProperty, value);
            }
        }
        public ColoringCellFactory ColoringCellFactory
        {
            get
            {
                return cellFactory;
            }
            set
            {
                cellFactory = value;
                OnPropertyChanged("ColoringCellFactory");
            }
        }

        public static readonly DependencyProperty FlexGridProperty = DependencyProperty.Register("FlexGrid", typeof(C1FlexGrid), typeof(InputManager), new PropertyMetadata(FlexGridChangeCallBack));
        private static void FlexGridChangeCallBack(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            if (e.NewValue != e.OldValue)
            {
                InputManager manager = (InputManager)d;
                manager.OnPropertyChanged("FlexGrid");
                manager.SetupFlexGrid();
            }
        }
        public void SetupFlexGrid()
        {
            if (this.FlexGrid == null)
                return;
            
            //Subscribe BeginningEdit event to show InkCanvas when cell is about to edit
            this.FlexGrid.BeginningEdit += FlexGrid_ShowInkCanvas;

            //Set the ColoredCellFactory for FlexGrid
            cellFactory = new ColoringCellFactory();
            FlexGrid.CellFactory = cellFactory;
            
        }
        
        public void SaveInkStrokes(StrokeCollection strokes)
        {
            using(MemoryStream ms = new MemoryStream())
            {
                strokes.Save(ms);
                var ink = new Microsoft.Ink.Ink();
                ink.Load(ms.ToArray());
                if (ink.Strokes.Count > 0)
                {
                    using (RecognizerContext recognizer = new RecognizerContext())
                    {
                        recognizer.Strokes = ink.Strokes;
                        RecognitionStatus status;
                        var result = recognizer.Recognize(out status);
                        if (status == RecognitionStatus.NoError)
                        {
                            FlexGrid[editingCell.Row, editingCell.Column] = result.ToString();
                            var color = result.Strokes[0].DrawingAttributes.Color;
                            AddColorInfo(editingCell, System.Windows.Media.Color.FromArgb(color.A, color.R, color.G, color.B));
                        }
                    }
                }                
            }            
        }

        private void AddColorInfo(CellRange editingCell, System.Windows.Media.Color color)
        {
            if (color != Colors.Black)
                ColoringCellFactory.ColoredCells[editingCell] = new SolidColorBrush(color);
            else
                ColoringCellFactory.ColoredCells.Remove(editingCell);
        }
        public void FlexGrid_ShowInkCanvas(object sender, CellEditEventArgs e)
        {
            this.FlexGrid.IsEnabled = false;
            editingCell = e.CellRange;

            InkInputDialog inkDialog = new InkInputDialog();
            if (inkDialog.ShowDialog() == true)
                SaveInkStrokes(inkDialog.Strokes);

            this.FlexGrid.IsEnabled = true;
            e.Cancel = true;
        }

        public event PropertyChangedEventHandler PropertyChanged;
        protected void OnPropertyChanged(string name)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(name));
        }
    }
    public class ColoringCellFactory : CellFactory
    {
        public Dictionary<CellRange, SolidColorBrush> ColoredCells = new Dictionary<CellRange, SolidColorBrush>();
        public override void CreateCellContent(C1FlexGrid grid, Border bdr, CellRange rng)
        {
            base.CreateCellContent(grid, bdr, rng);
            if (ColoredCells.Count > 0)
                FindAndStyleCell(bdr, rng);
        }
        private void FindAndStyleCell(Border bdr, CellRange rng)
        {
            var rangeExists = DoesRangeExist(rng);
            if (rangeExists)
            {
                try
                {
                    (bdr.Child as dynamic).Foreground = ColoredCells[rng];
                }
                catch { }
            }
        }
        public bool DoesRangeExist(CellRange rng)
        {
            return ColoredCells.Keys.Any(r => rng.IsSingleCell && rng.Row == r.Row && rng.Column == r.Column);            
        }
    }
}
